@@ -15,7 +15,7 @@ import sqlalchemy.exc
 from bs4 import BeautifulSoup
 from flask import Blueprint, request, current_app as app, jsonify, Response
 from flask_security import current_user, hash_password, verify_password
-from sqlalchemy import update, insert
+from sqlalchemy import update, insert, or_
 from werkzeug.utils import secure_filename
 import pika
 
@@ -27,6 +27,8 @@ from opentakserver.extensions import db, logger
 from opentakserver.models.CoT import CoT
 from opentakserver.models.EUD import EUD
 from opentakserver.models.Group import Group
+from opentakserver.models.GroupMission import GroupMission
+from opentakserver.models.GroupUser import GroupUser
 from opentakserver.models.Mission import Mission
 from opentakserver.models.MissionChange import MissionChange, generate_mission_change_cot
 from opentakserver.models.MissionContent import MissionContent
@@ -235,6 +237,13 @@ def get_mission_by_guid(mission_guid: str):
 
 @mission_marti_api.route('/Marti/api/missions')
 def get_missions():
+    cert = verify_client_cert()
+    if not cert:
+        return '', 401
+
+    username = cert.get_subject().commonName
+    user = app.security.datastore.find_user(username=username)
+
     password_protected = request.args.get('passwordProtected', False)
 
     tool = request.args.get('tool')
@@ -251,12 +260,23 @@ def get_missions():
 
     try:
         query = db.session.query(Mission)
-        if not password_protected:
-            query = query.where(Mission.password_protected is False)
-        if tool:
-            query = query.where(Mission.tool == tool)
+
+        # Let admins see all missions
+        if not user.has_role('administrator'):
+            group_filters = []
+            groups = db.session.execute(db.session.query(GroupUser).filter_by(user_id=user.id, direction=Group.IN)).scalars()
+            for group in groups:
+                group_filters.append(GroupMission.group_id == group.group.id)
+                logger.info(group.group.id)
+            if group_filters:
+                query = query.outerjoin(GroupMission).where(or_(*group_filters))
+
         missions = db.session.execute(query).scalars()
         for mission in missions:
+            if not password_protected and mission.password_protected:
+                continue
+            if tool and tool.lower() != "public" and mission.tool != tool:
+                continue
             response['data'].append(mission.to_json())
 
     except BaseException as e:
@@ -291,14 +311,22 @@ def all_invitations():
 @mission_marti_api.route('/Marti/api/missions/<mission_name>', methods=['PUT', 'POST'])
 def put_mission(mission_name: str):
     """ Used by the Data Sync plugin to create or change a mission """
+    cert = verify_client_cert()
+    if not cert:
+        return jsonify({"success": False, "error": "Missions are only supported on SSL connections"}), 400
+
+    username = cert.get_subject().commonName
+    user = app.security.datastore.find_user(username=username)
+
     new_mission = True
 
     if not mission_name or not request.args.get('creatorUid'):
         return jsonify({'success': False, 'error': 'Please provide a mission name and creatorUid'}), 400
 
     eud = db.session.execute(db.session.query(EUD).filter_by(uid=request.args.get('creatorUid'))).first()
-    if eud:
-        eud = eud[0]
+    if not eud:
+        return jsonify({'success': False, 'error': f"Invalid creatorUid: {request.args.get('creatorUid')}"}), 400
+    eud = eud[0]
 
     password = None
     mission = db.session.execute(db.session.query(Mission).filter_by(name=mission_name)).first()
@@ -329,10 +357,7 @@ def put_mission(mission_name: str):
         if new_mission:
             mission_role = MissionRole()
             mission_role.clientUid = mission.creator_uid
-            if eud:
-                mission_role.username = eud.user.username if eud.user else "anonymous"
-            else:
-                mission_role.username = "anonymous"
+            mission_role.username = eud.user.username if eud.user else "anonymous"
             mission_role.createTime = datetime.datetime.now(datetime.timezone.utc)
             mission_role.role_type = MissionRole.MISSION_OWNER
             mission_role.mission_name = mission_name
@@ -351,11 +376,17 @@ def put_mission(mission_name: str):
 
             event = generate_new_mission_cot(mission)
 
-            rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+            rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+            rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+            rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
             channel = rabbit_connection.channel()
-            channel.basic_publish(exchange="missions", routing_key="missions",
-                                  body=json.dumps(
-                                      {'uid': app.config.get("OTS_NODE_ID"), 'cot': tostring(event).decode('utf-8')}))
+
+            groups = db.session.execute(db.session.query(GroupUser).filter_by(user_id=user.id, enabled=True)).scalars()
+            for group in groups:
+                channel.basic_publish(exchange="groups", routing_key=f"{group.group.name}.{group.direction}", body=json.dumps({'uid': app.config.get("OTS_NODE_ID"), 'cot': tostring(event).decode('utf-8')}))
+
+            channel.close()
+            rabbit_connection.close()
     except sqlalchemy.exc.IntegrityError:
         # Mission exists, needs updating
         db.session.rollback()
@@ -439,11 +470,15 @@ def delete_mission(mission_name: str):
         db.session.delete(mission)
         db.session.commit()
 
-        rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+        rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+        rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+        rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
         channel = rabbit_connection.channel()
         channel.basic_publish(exchange="missions", routing_key="missions",
                               body=json.dumps({'uid': app.config.get("OTS_NODE_ID"),
                                                'cot': tostring(generate_mission_delete_cot(mission)).decode('utf-8')}))
+        channel.close()
+        rabbit_connection.close()
 
         return jsonify({'success': True})
     else:
@@ -533,10 +568,14 @@ def invite(mission_name: str, invitation_type: str, invitee: str):
     db.session.commit()
 
     event = generate_invitation_cot(mission, invitee)
-    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+    rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+    rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
     channel = rabbit_connection.channel()
     channel.basic_publish(exchange="dms", routing_key=invitee, body=json.dumps({"uid": app.config.get("OTS_NODE_ID"),
                                                                                 "cot": tostring(event).decode('utf-8')}))
+    channel.close()
+    rabbit_connection.close()
 
     return '', 200
 
@@ -592,7 +631,9 @@ def invite_json(mission_name: str):
         return jsonify({'success': False, 'error': f"Mission not found: {mission_name}"}), 404
     mission = mission[0]
 
-    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+    rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+    rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
     channel = rabbit_connection.channel()
 
     for invitee in invitees:
@@ -622,6 +663,9 @@ def invite_json(mission_name: str):
         channel.basic_publish(exchange="dms", routing_key=invitee['invitee'],
                               body=json.dumps({'uid': app.config['OTS_NODE_ID'], "cot": tostring(event).decode('utf-8')}))
 
+        channel.close()
+        rabbit_connection.close()
+
     return jsonify({'success': True})
 
 
@@ -714,10 +758,13 @@ def change_eud_role(mission_name: str):
         event = generate_invitation_cot(mission, role.clientUid)
         body = {'uid': app.config.get("OTS_NODE_ID"), 'cot': tostring(event).decode('utf-8')}
 
-        rabbit_connection = pika.BlockingConnection(
-            pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+        rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+        rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+        rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
         channel = rabbit_connection.channel()
         channel.basic_publish(exchange="dms", routing_key=client_uid, body=json.dumps(body))
+        channel.close()
+        rabbit_connection.close()
 
     # No new role provided, kick the EUD off the mission
     else:
@@ -729,10 +776,13 @@ def change_eud_role(mission_name: str):
         event = generate_invitation_cot(mission, client_uid, 't-x-m-r', delete=True)
         body = {'uid': app.config.get("OTS_NODE_ID"), 'cot': tostring(event).decode('utf-8')}
 
-        rabbit_connection = pika.BlockingConnection(
-            pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+        rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+        rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+        rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
         channel = rabbit_connection.channel()
         channel.basic_publish(exchange="dms", routing_key=client_uid, body=json.dumps(body))
+        channel.close()
+        rabbit_connection.close()
 
     return '', 200
 
@@ -797,8 +847,9 @@ def put_mission_keywords(mission_name):
 @mission_marti_api.route('/Marti/api/missions/guid/<mission_guid>/subscription', methods=['PUT'])
 def mission_subscribe(mission_name: str = None, mission_guid: str = None):
     """ Used by the Data Sync plugin to subscribe to a feed """
-
-    # ?uid=ANDROID-CloudTAK-administrator
+    cert = verify_client_cert()
+    username = cert.get_subject().commonName
+    user = app.security.datastore.find_user(username=username)
 
     if mission_name:
         mission = db.session.execute(db.session.query(Mission).filter_by(name=mission_name)).first()
@@ -813,6 +864,20 @@ def mission_subscribe(mission_name: str = None, mission_guid: str = None):
         return jsonify({'success': False, 'error': f"Cannot find mission {mission_guid}"}), 404
 
     mission = mission[0]
+    group_filters = []
+    groups = db.session.execute(db.session.query(GroupUser).filter_by(user_id=user.id)).scalars()
+    for group in groups:
+        group_filters.append(GroupMission.group_id == group.group_id)
+    if not group_filters:
+        # Default to the __ANON__ group
+        group_filters.append(GroupMission.group_id == 1)
+
+    query = db.session.query(GroupMission).filter_by(mission_name=mission_name)
+    query.where(or_(*group_filters))
+    mission_groups = db.session.execute(query).scalars()
+
+    if not mission_groups:
+        return jsonify({"success": False, "error": f"{username} and mission {mission_name} are not in the same group"}), 403
 
     response = {
         "version": "3", "type": "com.bbn.marti.sync.model.MissionSubscription", "data": {}, "nodeId": app.config.get("OTS_NODE_ID")
@@ -890,9 +955,13 @@ def mission_subscribe(mission_name: str = None, mission_guid: str = None):
             "role": role.to_json()['role'],
         }
 
-    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+    rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+    rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
     channel = rabbit_connection.channel()
     channel.queue_bind(queue=uid, exchange="missions", routing_key=f"missions.{mission_name}")
+    channel.close()
+    rabbit_connection.close()
 
     # Delete any invitations to this mission for this EUD
     invitations = db.session.execute(db.session.query(MissionInvitation).filter_by(mission_name=mission_name, client_uid=uid)).all()
@@ -928,9 +997,13 @@ def mission_unsubscribe(mission_name: str):
         db.session.delete(role[0])
         db.session.commit()
 
-    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+    rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+    rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
     channel = rabbit_connection.channel()
     channel.queue_unbind(queue=eud_uid, exchange="missions", routing_key=f"missions.{mission_name}")
+    channel.close()
+    rabbit_connection.close()
 
     return '', 200
 
@@ -977,29 +1050,22 @@ def create_log_entry():
     log_entry.keywords = request.json.get('keywords')
 
     db.session.add(log_entry)
-
-    response = {
-        'version': '3', 'type': '', 'messages': [], 'nodeId': app.config.get('OTS_NODE_ID'), 'data': [log_entry.to_json()]
-    }
-
-    mission_change = MissionChange()
-    mission_change.content_uid = log_entry.entry_uid
-    mission_change.isFederatedChange = False
-    mission_change.change_type = MissionChange.CHANGE
-    mission_change.timestamp = log_entry.dtg
-    mission_change.creator_uid = log_entry.creator_uid
-    mission_change.server_time = datetime.datetime.now(datetime.timezone.utc)
-    mission_change.mission_name = log_entry.mission_name
-
-    db.session.add(mission_change)
     db.session.commit()
 
-    change_cot = generate_mission_change_cot(log_entry.mission_name, mission[0], mission_change, cot_type='t-x-m-c-l')
+    response = {
+        'version': '3', 'type': 'com.bbn.marti.sync.model.LogEntry', 'nodeId': app.config.get('OTS_NODE_ID'), 'data': [log_entry.to_json()]
+    }
+
+    change_cot = log_entry.generate_cot()
     body = json.dumps({'uid': log_entry.creator_uid, 'cot': tostring(change_cot).decode('utf-8')})
 
-    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+    rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+    rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
     channel = rabbit_connection.channel()
     channel.basic_publish("missions", routing_key=f"missions.{log_entry.mission_name}", body=body)
+    channel.close()
+    rabbit_connection.close()
 
     return jsonify(response), 201
 
@@ -1050,6 +1116,11 @@ def upload_content():
         return jsonify({'success': False, 'error': 'File name cannot be blank'}), 400
 
     filename, extension = os.path.splitext(secure_filename(file_name))
+
+    # In some cases iTAK doesn't include the file extension. If the user agent includes iTAK and there's no extension we'll assume it's a zip
+    if not extension and "iTAK" in request.user_agent.string:
+        extension = "zip"
+
     if extension.replace('.', '').lower() not in app.config.get("ALLOWED_EXTENSIONS"):
         logger.error(f"{extension} is not an allowed file extension")
         return jsonify({'success': False, 'error': f'{extension} is not an allowed file extension'}), 400
@@ -1160,9 +1231,13 @@ def mission_contents(mission_name: str):
                 event = generate_mission_change_cot(mission_name, mission, mission_change, content=content)
 
                 body = json.dumps({'uid': mission_change.creator_uid, 'cot': tostring(event).decode('utf-8')})
-                rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+                rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+                rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+                rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
                 channel = rabbit_connection.channel()
                 channel.basic_publish("missions", routing_key=f"missions.{mission_name}", body=body)
+                channel.close()
+                rabbit_connection.close()
 
     if 'uids' in body:
         for uid in body['uids']:
@@ -1234,10 +1309,13 @@ def mission_contents(mission_name: str):
                 event = generate_mission_change_cot(mission_name, mission, mission_change, mission_uid=mission_uid)
 
                 body = json.dumps({'uid': mission_change.creator_uid, 'cot': tostring(event).decode('utf-8')})
-                rabbit_connection = pika.BlockingConnection(
-                    pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+                rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"), app.config.get("OTS_RABBITMQ_PASSWORD"))
+                rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+                rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
                 channel = rabbit_connection.channel()
                 channel.basic_publish("missions", routing_key=f"missions.{mission_name}", body=body)
+                channel.close()
+                rabbit_connection.close()
 
     db.session.commit()
 
@@ -1300,7 +1378,7 @@ def delete_content(mission_name: str):
 
         try:
             mission_content_mission = db.session.execute(db.session.query(MissionContentMission)
-                                                         .filter_by(mission_name=mission_name, mission_content_id=content.uid)).first()
+                                                         .filter_by(mission_name=mission_name, mission_content_id=content.id)).first()
             if mission_content_mission:
                 db.session.delete(mission_content_mission[0])
                 db.session.commit()
@@ -1312,10 +1390,14 @@ def delete_content(mission_name: str):
     event = generate_mission_change_cot(eud_uid, mission, mission_change, content=content, mission_uid=mission_uid, cot_event=cot_event)
     body = {'uid': app.config.get("OTS_NODE_ID"), 'cot': tostring(event).decode('utf-8')}
 
-    rabbit_connection = pika.BlockingConnection(
-        pika.ConnectionParameters(app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")))
+    rabbit_credentials = pika.PlainCredentials(app.config.get("OTS_RABBITMQ_USERNAME"),
+                                               app.config.get("OTS_RABBITMQ_PASSWORD"))
+    rabbit_host = app.config.get("OTS_RABBITMQ_SERVER_ADDRESS")
+    rabbit_connection = pika.BlockingConnection(pika.ConnectionParameters(host=rabbit_host, credentials=rabbit_credentials))
     channel = rabbit_connection.channel()
     channel.basic_publish(exchange="missions", routing_key=f"missions.{mission_name}", body=json.dumps(body))
+    channel.close()
+    rabbit_connection.close()
 
     db.session.add(mission_change)
     db.session.commit()
@@ -1325,9 +1407,6 @@ def delete_content(mission_name: str):
 
 @mission_marti_api.route('/Marti/api/missions/<mission_name>/contents/missionpackage', methods=['PUT'])
 def add_content(mission_name):
-    logger.info(request.headers)
-    logger.info(request.args)
-    logger.info(request.data)
     client_uid = request.args.get('clientUid')
     if 'Authorization' not in request.headers or not verify_token():
         return jsonify({'success': False, 'error': 'Missing or invalid token'}), 401
