@@ -7,6 +7,7 @@ import uuid
 from urllib.parse import urlparse
 
 from ffmpeg import FFmpeg
+from flask_ldap3_login import AuthenticationResponseStatus
 from sqlalchemy import update
 from werkzeug.datastructures import ImmutableMultiDict
 
@@ -17,7 +18,7 @@ from flask import current_app as app, request, Blueprint, jsonify
 from flask_security import auth_required, current_user, verify_password
 from flask_security.utils import parse_auth_token
 
-from opentakserver.extensions import logger, db
+from opentakserver.extensions import logger, db, ldap_manager
 from opentakserver.models.VideoStream import VideoStream
 from opentakserver.forms.MediaMTXPathConfig import MediaMTXPathConfig
 from opentakserver.models.VideoRecording import VideoRecording
@@ -384,11 +385,29 @@ def external_auth():
                 else:
                     return '', 401
 
-    user = app.security.datastore.find_user(username=username)
-    if not user:
-        return '', 401
+    auth_success = False
 
-    if user and verify_password(password, user.password):
+    # LDAP Auth
+    if app.config.get("OTS_ENABLE_LDAP"):
+        result = ldap_manager.authenticate(username, password)
+        if result.status == AuthenticationResponseStatus.success:
+            # Keep this import here to avoid a circular import when OTS is started
+            from opentakserver.blueprints.ots_api.ldap_api import save_user
+
+            save_user(result.user_dn, result.user_id, result.user_info, result.user_groups)
+            auth_success = True
+        else:
+            return '', 401
+    # Flask-Security auth
+    else:
+        user = app.security.datastore.find_user(username=username)
+        if not user:
+            return '', 401
+        if not verify_password(password, user.password):
+            return '', 401
+        auth_success = True
+
+    if auth_success:
         if action == 'publish':
             logger.debug("Publish {}".format(request.json.get('path')))
             v = VideoStream()
