@@ -1,4 +1,5 @@
 import colorlog
+from flask_ldap3_login import LDAP3LoginManager
 from flask_migrate import Migrate
 from flask_sqlalchemy import SQLAlchemy
 from flask_socketio import SocketIO
@@ -17,3 +18,5 @@ db = SQLAlchemy(model_class=Base)
 socketio = SocketIO(async_mode='gevent')
 
 migrate = Migrate()
+
+ldap_manager = LDAP3LoginManager()
\ No newline at end of file
