@@ -13,4 +13,7 @@ class Role(db.Model, fsqla.FsRoleMixin):
             'description': self.description,
             'permissions': self.permissions,
             'update_timestamp': self.update_datetime,
-        }
\ No newline at end of file
+        }
+
+    def __eq__(self, other):
+        return self.name == other or self.name == getattr(other, "name", None)
